@@ -37,6 +37,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+#include <dbus/dbus-glib.h>
 
 #include "hcp-window.h"
 #include "hcp-program.h"
@@ -77,7 +78,25 @@
 #define HCP_SIM_SCRIPT         "/usr/bin/startup-pin-query"
 
 #define HCP_WITH_ROS           1
-#define HCP_WITH_CUD           0
+#define HCP_WITH_CUD           1
+#define HCP_PORTRAIT_DEBUG     0 /* Only for debugging purposes */
+
+#if HCP_WITH_SIM
+#define	 HCP_SIMLOCK_NAME          "com.nokia.phone.SIM"
+#define	 HCP_SIMLOCK_PATH          "/com/nokia/phone/SIM"
+#define	 HCP_SIMLOCK_INTERFACE     "Phone.Sim"
+#define  HCP_SIMLOCK_CHECK_METHOD  "read_simlock_status"
+
+enum simlock_restriction_status{
+  HCP_SIMLOCK_RESTRICTED = 2,
+  HCP_SIMLOCK_RESTRICTION_ON = 3,
+  HCP_SIMLOCK_RESTRICTION_PENDING = 4,
+  HCP_SIMLOCK_STATE_NOT_INITIALIZED = 5,
+  HCP_SIMLOCK_NO_SERVICE = 6,
+  HCP_SIMLOCK_NOT_READY = 7,
+  HCP_SIMLOCK_ERROR = 8
+};
+#endif /* if HCP_WITH_SIM */
 
 #define HCP_WINDOW_GET_PRIVATE(object) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), HCP_TYPE_WINDOW, HCPWindowPrivate))
@@ -89,12 +108,22 @@ struct _HCPWindowPrivate
   HCPApp         *focused_item;
   HCPAppList     *al;
   GtkWidget      *view;
+  GtkWidget      *align;
 
   /* For state save data */
   gchar          *saved_focused_filename;
   gint            scroll_value;
+
+#if HCP_WITH_SIM
+  GtkWidget      *mi_simlock;
+#endif
 };
 
+#if HCP_WITH_SIM
+static void
+hcp_window_check_simlock (HCPWindow *window);
+#endif
+
 static void 
 hcp_window_enforce_state (HCPWindow *window)
 {
@@ -410,6 +439,24 @@ hcp_window_sim_unlock (GtkWidget *widget, HCPWindow *window)
   return TRUE;
 }
 #endif
+#if HCP_PORTRAIT_DEBUG
+static gboolean
+hcp_portrait_debug (GtkWidget *widget, HCPWindow *window)
+{
+    gint   flag = 0;
+    static gboolean portrait = FALSE;
+
+    portrait = ! portrait;
+
+    if (portrait)
+        flag = HILDON_PORTRAIT_MODE_REQUEST;
+
+    g_debug ("Portrait mode = %s", portrait ? "true" : "false");
+    hildon_gtk_window_set_portrait_flags (GTK_WINDOW (window), flag);
+
+    return TRUE;
+}
+#endif
 #endif /* ifdef MAEMO_TOOLS */
 
 static void 
@@ -542,7 +589,75 @@ _expose_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
   g_signal_handler_disconnect(G_OBJECT(data), program->handler_id);
   return FALSE;
 }
+#if HCP_WITH_SIM
+static void
+_simlock_cb (DBusGProxy* proxy, DBusGProxyCall* call, HCPWindow *window)
+{
+  gint ret;
+  GError *err = NULL;
 
+  HCPWindowPrivate *priv;
+  GtkWidget *mi;
+
+  g_return_if_fail (window);
+  g_return_if_fail (HCP_IS_WINDOW (window));
+
+  priv = window->priv;
+
+  mi = priv->mi_simlock;
+  g_return_if_fail (mi);
+  g_return_if_fail (GTK_IS_WIDGET (mi));
+
+  if (!dbus_g_proxy_end_call (proxy, call,
+                          &err, G_TYPE_INT,
+                          &ret, G_TYPE_INVALID)) {
+    g_warning ("unlock - communication error: %s\n", err->message);
+    g_error_free (err);
+  } else {
+    if ( ret != HCP_SIMLOCK_STATE_NOT_INITIALIZED &&
+         ret != HCP_SIMLOCK_NO_SERVICE &&
+         ret != HCP_SIMLOCK_NOT_READY &&
+         ret != HCP_SIMLOCK_ERROR ) {
+      /* show unlock button */
+      gtk_widget_set_no_show_all (mi, FALSE);
+      gtk_widget_show_all (mi);
+    } else {
+      /* hide unlock button */
+      gtk_widget_hide (mi);
+      gtk_widget_set_no_show_all (mi, TRUE);
+    }
+  }
+}
+
+static void
+hcp_window_check_simlock (HCPWindow* window)
+{
+  g_return_if_fail (window);
+  g_return_if_fail (HCP_IS_WINDOW (window));
+
+  DBusGConnection *conn;
+  GError *err = NULL;
+
+  conn = dbus_g_bus_get (DBUS_BUS_SYSTEM, &err);
+  if (!conn) {
+    g_warning ("Could not connect to dbus: %s\n", err->message);
+    g_error_free (err);
+  } else {
+    DBusGProxy *proxy;
+    proxy = dbus_g_proxy_new_for_name (conn,
+                                       HCP_SIMLOCK_NAME,
+                                       HCP_SIMLOCK_PATH,
+                                       HCP_SIMLOCK_INTERFACE);
+
+    dbus_g_proxy_begin_call (proxy,
+                             HCP_SIMLOCK_CHECK_METHOD,
+                             (DBusGProxyCallNotify)_simlock_cb,
+                             window,
+                             NULL,
+                             G_TYPE_INVALID);
+  }
+}
+#endif /* if HCP_WITH_SIM */
 static void 
 hcp_window_construct_ui (HCPWindow *window)
 {
@@ -643,7 +758,24 @@ hcp_window_construct_ui (HCPWindow *window)
 
   g_signal_connect (mi, "clicked",
                     G_CALLBACK (hcp_window_sim_unlock), window);
+
+  gtk_widget_set_no_show_all(mi,TRUE);
+  priv->mi_simlock = mi;
+  hcp_window_check_simlock (window);
 #endif /* if HCP_WITH_SIM */
+
+#if HCP_PORTRAIT_DEBUG
+  mi = hildon_button_new_with_text (HILDON_SIZE_AUTO_WIDTH |
+									HILDON_SIZE_FINGER_HEIGHT, 
+									HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+									"Portrait debug", NULL);
+
+  hildon_app_menu_append (menu, GTK_BUTTON (mi));
+
+  g_signal_connect (mi, "clicked",
+                    G_CALLBACK (hcp_portrait_debug), window);
+#endif
+
 #endif /* ifdef MAEMO_TOOLS */
   
   gtk_widget_show_all (GTK_WIDGET (menu));
@@ -659,20 +791,34 @@ hcp_window_construct_ui (HCPWindow *window)
 
   gtk_container_add (GTK_CONTAINER (window), scrolled_window);
 
-  GtkWidget *align = gtk_alignment_new (0,0,0,0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT(align),0,0,
-     HILDON_MARGIN_DOUBLE,HILDON_MARGIN_DOUBLE);
+  window->priv->align = gtk_alignment_new (0,0,0,0);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (window->priv->align),
+                             HILDON_MARGIN_DOUBLE, 
+                             0,
+                             HILDON_MARGIN_DOUBLE,
+                             HILDON_MARGIN_DOUBLE);
 
-  /* margins are HILDON_MARGIN_DOUBLE, allocate rest of the space to view*/
-  gtk_widget_set_size_request (GTK_WIDGET (priv->view), 760, -1);
-  gtk_container_add (GTK_CONTAINER(align), GTK_WIDGET(priv->view));
+  gtk_container_add (GTK_CONTAINER(window->priv->align),
+                     GTK_WIDGET(priv->view));
 
-  /*gtk_container_add (GTK_CONTAINER(view), align); */
   hildon_pannable_area_add_with_viewport (
           HILDON_PANNABLE_AREA (scrolled_window),
-          align);
+          window->priv->align);
 }
 
+static void
+hcp_window_size_request (HCPWindow          *window,
+                         GtkRequisition     *requisition,
+                         gpointer            user_data)
+{
+    if (G_UNLIKELY (! window->priv->view))
+        return;
+
+    gtk_widget_set_size_request (window->priv->align,
+                                 requisition->width - HILDON_MARGIN_DEFAULT,
+                                 -1);
+}                  
+
 static void
 hcp_window_init (HCPWindow *window)
 {
@@ -686,11 +832,20 @@ hcp_window_init (HCPWindow *window)
   priv->focused_item = NULL;
   priv->saved_focused_filename = NULL;
   priv->scroll_value = 0;
-
+#if HCP_WITH_SIM
+  priv->mi_simlock = NULL;
+#endif
   priv->al = g_object_ref (program->al);
 
   hcp_window_retrieve_configuration (window);
 
+  /* Turn on portrait mode support flag */
+  /* TODO FIXME XXX Turn on this flag when applets ready... */
+#if 0
+  hildon_gtk_window_set_portrait_flags (GTK_WINDOW (window),
+                                        HILDON_PORTRAIT_MODE_SUPPORT);
+#endif
+
   priv->view = hcp_app_view_new ();
   g_signal_connect (G_OBJECT (priv->view), "focus-changed",
                     G_CALLBACK (hcp_window_app_view_focus_cb), window);
@@ -698,6 +853,9 @@ hcp_window_init (HCPWindow *window)
   g_signal_connect (G_OBJECT (priv->al), "updated",
                     G_CALLBACK (hcp_window_app_list_updated_cb), window);
 
+  g_signal_connect (G_OBJECT (window), "size-request",
+                    G_CALLBACK (hcp_window_size_request), NULL);
+
   hcp_window_retrieve_state (window);
 
   hcp_window_construct_ui (window);
@@ -792,3 +950,4 @@ hcp_window_close (HCPWindow *window)
 
   hcp_window_quit (NULL, window);
 }
+
